#ifndef REGISTERS_H_
#define REGISTERS_H_

#define DRIVER_OUTPUT_REGISTER 0xFFAAu
#define DRIVER_INPUT_REGISTER  0XFFABu
#define DRIVER_PERIPHERAL_ENABLE_REG 0xFFACu
#define DRIVER_PERIPHERAL_INITIALIZE_REG 0xFFACu

#define HARDWARE_VERSION_REGISTER 0xFF00u
#define HARDWARE_REV_A 0x00u
#define HARDWARE_REV_B 0x01u

#endif /* REGISTERS_H_ */
